!+
! PROGRAM AeroelasticDivergence
! ---------------------------------------------------------------------------
! PURPOSE - Compute the aeroelastic divergence characteristics of unguided, 
!  slender body, multi-stage launch vehicles

!     TGB 800  PN 8630F       
! AUTHORS - LESSIE HUNTER FOR CLARENCE YOUNG ??
!  Ralph L. Carmichael, Public Domain Aeronautical Software


! REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
! 11Aug97  0.1   RLC   Acquisition of COSMIC Program LAR-12004
! 20Aug97  0.2   RLC   Removed tabs from source code
! 31Aug97  0.3   RLC   Converted all source to free-format Fortran 90
!   ?      0.4   RLC   Moved all subroutines to a module. Names on END records 
! 11Dec09  0.5   RLC   Converted some Hollerith to character strings   
! 04Feb10  0.6   RLC   All Hollerith to character strings


! NOTES - Original program description from COSMIC...
! The primary function of this computer program is the calculation of the
! divergence dynamic pressure and associated divergence modal characteristics 
! of unguided, slender-body, multistage launch vehicles. The divergence 
! dynamic pressure is obtained as the non-trivial solution to a homogenous 
! stability equation using matrix recurrence techniques. Provision is made for 
! modulating the distributed lift curve coefficient slope function and the 
! stiffness function. The program also includes an option for calculating a 
! generalized static margin which approximates the degeneration in rigid-body 
! static margin due to aeroelasticity effects. Evaluated equations are also 
! programmed to allow for the exclusion of the effect of aerodynamic crossflow 
! resulting from vehicle angular velocities if desired. Other physical and 
! aerodynamic properties calculated include total mass, center of mass, moments 
! of inertia in pitch about the reference station, total aerodynamic lift curve 
! slope, static aerodynamic center of pressure, rigid body static margin, and 
! short period frequency. Input to the program is via the Fortran NAMELIST 
! option with output printed.




!+
MODULE AeroelasticDivergenceProcedures

CONTAINS

!!!      OVERLAY(LINK,1,0)
!!!      PROGRAM ENPUT
      SUBROUTINE Enput()
! PURPOSE -     THIS SECTION READS VARIABLE AND CONSTANT INPUT



!!!      DIMENSION HEAD(12)
      CHARACTER(LEN=80):: head
      DIMENSION PARAM(4800), OUT(8), X(600), MASS(600), EI(600)
      DIMENSION DCLA(600), DCDA(600), K1(600), K2(600), II(7)
      DIMENSION DIA(600)
      REAL K1,K2,MASS
      INTEGER:: errCode
      EQUIVALENCE (II(1),I1), (II(2),I2), (II(3),I3), (II(4),I4)
      EQUIVALENCE (II(5),I5), (II(6),I6), (II(7),I7), (PARAM(1),X)
      NAMELIST /CNTRL/ JNT,KIT,KCASE
      NAMELIST /IDENT/ NH
      NAMELIST /ENPAR/ JNCOL,JXSTA,EXPON,FCTRMS,DCLA,DCDA,K1,K2,DIA
      NAMELIST /AERDIV/ X,MASS,EI
      NAMELIST /MOVER/ INSPAN/LIMB/NLO,NUP,CAPPA

!          READ IN CONTROLS FOR FLOW OF PROGRAM
!     CONTROL       1                             2
!     JNT      WITHOUT JOINTS                WITH JOINTS
!     KIT      ITERATE                       DONT ITERATE


    1 READ (5,CNTRL,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading CNTRL'
        WRITE(6,*) 'End of file reading CNTRL'
        STOP
      END IF
      WRITE (6,CNTRL)

!          CASE IDENTIFICATION
!          NO OF LINES
      READ (5,IDENT,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading IDENT'
        WRITE(6,*) 'End of file reading IDENT'
        STOP
      END IF

      WRITE (6,16)
      DO I=1,NH
        READ(5,'(A)') HEAD
       WRITE(6,'(1X,A)') TRIM(HEAD)
      END DO

!          READ INPUT PARAMETERS
!          NO. OF COLUMNS,NO. OF STATIONS
!          MULTIPLYING FACTOR FOR EI,INPUT DATA BY COLUMNS
      READ (5,ENPAR,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading ENPAR'
        WRITE(6,*) 'End of file reading ENPAR'
        STOP
      END IF

      READ (5,AERDIV,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading AERDIV'
        WRITE(6,*) 'End of file reading AERDIV'
        STOP
      END IF

      DO 4 I=1,JXSTA
      DO 3 J=1,7
    3 II(J)=JXSTA*(J)+I
      PARAM(I1)=MASS(I)
      PARAM(I2)=EI(I)
      PARAM(I3)=DCLA(I)
      PARAM(I4)=DCDA(I)
      PARAM(I5)=K1(I)
      PARAM(I6)=K2(I)
    4 PARAM(I7)=DIA(I)

!     MULTIPLY MASS BY FACTOR
      IM=JXSTA+1
      IE=2*JXSTA

      DO 5 J=IM,IE
    5 PARAM(J)=PARAM(J)*FCTRMS
!          MULTIPLY EI BY EXPONENT
      IM=2*JXSTA+1
      IE=3*JXSTA
      DO 6 J=IM,IE
    6 PARAM(J)=PARAM(J)*EXPON

      GO TO (7,10), JNT

!          NO JOINTS

    7 REWIND 2
      WRITE (2) JNCOL,JXSTA,JNT,KIT,KCASE,I7,(PARAM(I),I=1,I7)
!          NO. OF COLUMNS,NO OF STATIONS
      WRITE (6,13) JNCOL,JXSTA
!         STATION,X,M,EI,DC(N)ALPHA /DX,DC(D)ALPHA/DX,K1,K2,DIAMETER
      WRITE (6,14)
      DO K=1,JXSTA
        IC=K-1
        DO I=1,8
          ID=(I-1)*JXSTA+K
          OUT(I)=PARAM(ID)
        END DO
        WRITE (6,15) IC,(OUT(J),J=1,8)
      END DO
      WRITE(6,*) "Enput completed."
      RETURN                                    ! this is the way out


!          ADDITIONAL INPUT FOR JOINT EFFECTS
!          INTERVALS OF SPAN IN EI
   10 READ (5,MOVER)
      WRITE (6,19) INSPAN
      DO 12 J=1,INSPAN
!          LOWER AND UPPER LIMITS OF EACH SPAN
      READ (5,LIMB)
      WRITE (6,LIMB)
      KXN=NLO+1
      KXNP1=NUP+1
      KEIN=2*JXSTA+KXN
      KX2=KXN+1
      KY2=KEIN+1
      EIGRL=0.0
      XDEL=ABS(PARAM(KXNP1)-PARAM(KXN))
      DO 11 I=KX2,KXNP1
      H=ABS(PARAM(I)-PARAM(I-1))
      EIGRL=EIGRL+H/PARAM(KEIN)
   11 KEIN=KEIN+1
      EIQV=XDEL/(CAPPA+EIGRL)
      DO 12 K=KXN,KXNP1
      JE=2*JXSTA+K
   12 PARAM(JE)=EIQV

      WRITE(6,*) 'MOVER and LIMB input read in.'
      GO TO 7
!
   13 FORMAT (/' COLUMNS    ',I4/' STATIONS   ',I4)
   14 FORMAT(/2X,'STA',9X,'X',13X,'M(X)',13X,'EI',13X,'DCLA',13X, &
        'DCDA',13X,'K1',14X,'K2',11X,'DIAMETER')
   15 FORMAT(I6, 7ES16.8, ES15.7)
   16 FORMAT(1H1)
!   17 FORMAT(12A6)   ! never used
!   18 FORMAT(1X12A6)  ! never used
   19 FORMAT(/' EI EQUIVALENT COMPUTED FOR ',I2,' JOINTS')
      END Subroutine Enput

!!!      OVERLAY(LINK,2,0)
!!!      PROGRAM R0863
      SUBROUTINE R0863()
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMMN
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
      INTEGER:: errCode
      NAMELIST /ACASE/ JCASE
      NAMELIST /CSTANT/ QCON,TCON,EMU,PZER,YZER,SMALK,TAU,ELMBDA,EN,SCDB
      NAMELIST /NVAL/ COND
      NAMELIST /SUME/ KSUB,ASSUME,STEP,EPS1,EPS2,EPS3
      NAMELIST /KASPAN/ JLO,JUP,ACTK
!
!     TGB 800  PN 8630F       LESSIE HUNTER FOR CLARENCE YOUNG
!     AEROELASTIC DIVERGENCE PROBLEM
!
!     MAIN PROGRAM
!
      GO TO (1,5), N
    1 N=2

      DO 2 LLL=1,8
      DO 2 LLM=1,8
      D1(LLL,LLM)=0.0
    2 CKAY(LLL,LLM)=0.0

      DO 3 LLL=1,8
      CHIZER(LLL)=0.0
      CHIN1(LLL,1)=0.0
      CHIN2(LLL,1)=0.0
      CHNP11(LLL,1)=0.0
      CHNP12(LLL,1)=0.0
      CHIA(LLL,1)=0.0
      CHIB(LLL,1)=0.0
      D2(LLL,1)=0.0
    3 D2(LLL,1)=0.0

!              NO OF CASES
      READ(5,ACASE,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading ACASE'
        WRITE(6,*) 'End of file reading ACASE'
        STOP
      END IF
    4 RETURN
    5 REWIND 2
      READ (2) JNCOL,JXSTA,JNT,KIT,KCASE,I7,(PARAM(I),I=1,I7)
!
!          ENTRY TO BEGIN A CASE
!          CONTROL CARD,IDENTIFICATION,DATA
!          NO OF COLUMNS TO ROWS
      JCR=JNCOL-1
!          NO. OF INPUT VALUES TO ROWS
      JNEL=JCR*JXSTA

!          READ INPUT CONSTANTS
      READ (5,CSTANT,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading CSTANT'
        WRITE(6,*) 'End of file reading CSTANT'
        STOP
      END IF

!          INPUT CONSTANTS
      WRITE (6,28) QCON,TCON,EMU,PZER,YZER,SMALK,TAU,ELMBDA,EN,SCDB
!          FIND INTEGRALS FOR COMPUTING P
      CALL PEA (1)
!
!          START ROWS
      JK=JNCOL*JXSTA
!          PUT NECESSARY COLUMNS IN ROWS
      CALL ROWS
!
!
!          DETERMINE SYSTEM
      CALL SYST (1)
      LSUB=1
!
!          READ INITIAL VALUES FOR THREE SOLUTIONS
!
!          INITIAL VALUES(2 SOLUTIONS)
!         V(A),ALPHA(A),M(A),1/R
      READ (5,NVAL,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading NVAL'
        WRITE(6,*) 'End of file reading NVAL'
        STOP
      END IF
      WRITE (6,NVAL)
!          SAVE INITIAL VALUES
      DO 6 J=1,8
    6 ACOND(J)=COND(J)
!
!          ENTRY TO BEGIN A SUBCASE
!             SUBCASE 1      SEARCH FOR DIVERGENCE Q
!             SUBCASE 2      SEARCH FOR CRITICAL T
!             SUBCASE 3      SEARCH FOR CRITICAL PA
!             SUBCASE 4      SEARCH FOR DIVERGENCE K1
!          READ INITIAL VALUES FOR SUBCASE ITERATION
!
!          ASSUMED VALUES FOR SUBCASE ITERATION
!     SUBCASE
!     1 QDIV   Q,DELQ,EPS1,EPS2,EPS3
!     2 TCR    T,DELT,ETC.
!     3 PZCR   P(A),DELP(A) ETC
!     4 K1DIV  Q,DELQ,ETC.)))
!
    7 READ (5,SUME,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading SUME'
        WRITE(6,*) 'End of file reading SUME'
        STOP
      END IF
      WRITE (6,SUME)
!
!          SEE IF ITERATION IS FOR K1
      GO TO (9,9,9,8), KSUB
!
!          READ LIMITS OF INTERVAL TO BE CHANGED(STATION NO)
!          INTERVAL TO BE CHANGED FOR K1 ITERATION ONLY
!          READ LOWER STATION NO.,UPPER STATION NO.
!          ACTUAL K
    8 READ (5,KASPAN,IOSTAT=errCode)
      IF (errCode < 0) THEN
        WRITE(*,*) 'End of file reading KASPAN'
        WRITE(6,*) 'End of file reading KASPAN'
        STOP
      END IF
      WRITE (6,KASPAN)
!          K(FIN)=K(ACTUAL)
      FINK=ACTK
      KLO=JCR*JLO+JXSTA+5
      KUPP=JCR*JUP+JXSTA+5
    9 GO TO (11,10), KIT
!          NO ITERATION--COMPUTES THROUGH FINAL RUN ONLY
   10 TEMP=ASSUME
      ROOT=ASSUME
      KIF=1
      GO TO 14
!
!
!          SETUP SUBCASE FOR ITERATION
   11 CALL QTPK (1)
!          SET FOR INITIAL RUN
      KIF=1
!
!              INITIAL RUN
!          COMPUTE P
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
      CALL FIND
!          SEE IF ROOT IS FOUND
      GO TO (13,12), IF
!          NO ROOT FOUND IN SEARCH ROUTINE,DETOUR
   12 GO TO 17
   13 TEMP=ROOT
   14 CALL QTPK (2)
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
      MSUB=1
!          COMPUTE INITIAL VALUES FOR FINAL RUN
      CALL SYST (4)
!          SET FOR FINAL RUN
      KIF=2
!
!          FINAL RUN
!         PRINT DIVERGENCE VALUES,1/R
      CALL QTPK (3)
      IF (MSUB.NE.4) GO TO 16
!          FIND Q DIV FOR KDIV FOUND
      DO 15 J=1,8
   15 COND(J)=ACOND(J)
      GO TO 11
!          HEADINGS
   16 CALL OUTPUT (1)
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
!          WRITE P AND Y
      CALL OUTPUT (3)
!          CHECK FOR ALL SUBCASES
   17 IF (KCASE-1) 18,21,19
   18 WRITE (6,25)
      STOP
!          NO-GO FOR NEXT
   19 KCASE=KCASE-1
      DO 20 J=1,8
   20 COND(J)=ACOND(J)
      WRITE (6,27)
      GO TO 7
!          TO INTEGRAL SUBROUTINE
   21 CALL EXTRA
!          CHECK FOR ALL CASES
      IF (JCASE-1) 22,24,23
   22 WRITE (6,26)
      STOP
!          NO- GO FOR NEXT
   23 JCASE=JCASE-1
      GO TO 4
!
!          YES-EXIT FROM PROGRAM
   24 STOP
!
   25 FORMAT (' SUBCASE COUNT')
   26 FORMAT (' CASE COUNT')
   27 FORMAT (///)
   28 FORMAT (/' Q      ',E16.8, '     T      ', ES18.8, '      U     ', E16.8/ &
     &/,' P(A)   ', ES16.8, '     Y(A)   ', ES18.8, '     K      ',ES16.8// &
     & ' TAU    ', ES16.8, '     LAMBDA ', ES18.8, '     N      ', ES16.8/ &
     & ' SCD(B) ', ES16.8)
END Subroutine R0863

      SUBROUTINE ROWS()
!     THIS SECTION PUTS NECESSARY COLUMNS IN ROWS. X IS LEFT IN COLS.
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      L=JK+1
      DO 1 M=1,JXSTA
      DO 1 J=1,JCR
      K=(J*JXSTA)+M
      PARAM(L)=PARAM(K)
    1 L=L+1
!          SHIFT TO FOLLOW X
      DO 2 J=1,JNEL
      L1=JK+J
      K1=JXSTA+J
    2 PARAM(K1)=PARAM(L1)
      RETURN
      END Subroutine Rows

      SUBROUTINE SYST (N)
!     THIS SECTION DETERMINES THE SYSTEM USED
!
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      GO TO (1,2,3,4), N
    1 CALL FREE (1)
      RETURN
    2 CALL FREE (2)
      RETURN
    3 CALL FREE (3)
      RETURN
    4 CALL FREE (4)
      RETURN
      END Subroutine Syst

      SUBROUTINE FREE (N)
!     THIS SECTION CONTAINS INFORMATION RELATIVE TO THE
!     FREE FREE THRUSTING SYSTEM
!
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      GO TO (1,2,6,7), N
!          DETERMINE CASE AND CONDITIONS NEEDED
!              A CONDITION(SOLUTION 1)
!
    1 KCOND(1)=2
!              B CONDITION(SOLUTION 2)
      KCOND(2)=4
      RETURN
!
!        CHI(A)   LOWER BOUNDARY
!         CLEAR CHI(A)  STORAGE
    2 DO 3 I=1,8
    3 CHIZER(I)=0.0
      GO TO (4,5), JBRAN
!          A SOLUTION
    4 CHIZER(1)=ALZER*ROWN(5)*ROWN(3)*COMPQ
      CHIZER(2)=ALZER*COMPT
      CHIZER(4)=ALZER
      CHIZER(5)=-SHDCMA(1)+SCDB*COMPQ
      RETURN
!          B SOLUTION
    5 CHIZER(1)=-R*ROWN(1)*EMU**2
      CHIZER(3)=-R*ELMBDA
      CHIZER(5)=-CHIZER(3)*(-COMPT+SCDB*COMPQ)*PARAM(1)
      CHIZER(8)=R
      RETURN
!
!          SOLVE STABILITY EQUATION
    6 FMEGA=CHNP11(2,1)*CHNP12(6,1)
      FMEGB=CHNP11(6,1)*CHNP12(2,1)
      FMEG=FMEGA-FMEGB
      WRITE (6,8) FMEG
      RETURN
!          INITIAL VALUES FOR FINAL RUN
!         COMPUTE ALPHA (A)=-V2/V1 FOR FINAL RUN
    7 COND(2)=-CHNP12(2,1)/CHNP11(2,1)*EN
      COND(8)=EN
      RETURN
!
    8 FORMAT (' DETERMINANT =',ES16.8)
      END Subroutine Free

      SUBROUTINE QTPK (N)
!     THIS SECTION SELECTS VALUES NEEDED FOR A PARTICULAR SUBCASE.
!
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
!
      GO TO (1,7,19), N
!
!          DETERMINE SUBCASE
!          SELECT Q,T,P(A),OR K1 TO FIND CHARACTERISTIC VALUES
    1 GO TO (2,3,4,5), KSUB
!          ITERATE FOR QDIV
!          T AND P(A) ARE CONSTANT
    2 COMPT=TCON
      COMPPZ=PZER
      TEMP2=STEP
!          ASSUMED Q
      COMPQ=ASSUME
      TEMP=COMPQ
      RETURN
!
!          ITERATE FOR T
!          Q AND P(A) ARE CONSTANT
    3 COMPQ=QCON
      COMPPZ=PZER
      TEMP2=STEP
!          ASSUMED T
      COMPT=ASSUME
      TEMP=COMPT
      RETURN
!
!          ITERATE FOR P(A)CR
!          Q AND T ARE CONSTANT
    4 COMPQ=QCON
      COMPT=TCON
      TEMP2=STEP
!          ASSUMED P(A)
      COMPPZ=ASSUME
      TEMP=COMPPZ
      RETURN
!          ITERATE FOR K1DIV
!          Q,T,P(A) ARE CONSTANT
    5 COMPQ=QCON
      COMPPZ=PZER
      COMPT=TCON
!          ASSUMED K1
      TEMP=ASSUME
!          MOVE TO INTERVAL
      KLO=JCR*JLO+JXSTA+5
      KUPP=JCR*JUP+JXSTA+5
      DO 6 I=KLO,KUPP,JCR
    6 PARAM(I)=ASSUME
      TEMP2=STEP
      RETURN
!
!          MOVE  Q,T,P(A),OR K1 TO COMPUTE
    7 GO TO (8,11,14,17), KSUB
    8 COMPQ=TEMP
      GO TO (10,9), KIT
!          NO ITERATION(FINAL RUN)  QDIV GIVEN
    9 COMPT=TCON
      COMPPZ=PZER
   10 RETURN
   11 COMPT=TEMP
      GO TO (13,12), KIT
!          NO ITERATION(FINAL RUN)   TCR GIVEN
   12 COMPQ=QCON
      COMPPZ=PZER
   13 RETURN
!
   14 COMPPZ=TEMP
      GO TO (16,15), KIT
!          NO ITERATION(FINAL RUN)   P(A)  GIVEN
   15 COMPQ=QCON
      COMPT=TCON
   16 RETURN
!
!         NO ITERATION(FINAL RUN)   K1DIV GIVEN AND QDIV GIVEN
   17 DO 18 I=KLO,KUPP,JCR
   18 PARAM(I)=TEMP
      COMPQ=QCON
      COMPT=TCON
      COMPPZ=PZER
      RETURN
!
!          OUTPUT
!          HOMOGENEOUS (CRITICAL VALUES,1/R)
   19 GO TO (20,23,26,27), KSUB
!
   20 QDIV=ROOT
      IF (QDIV) 22,21,22
   21 WRITE (6,32)
      STOP
   22 Q1=QCON/QDIV
      WRITE (6,28) COND(8),QDIV,Q1
      RETURN
!
   23 TCR=ROOT
      IF (TCR) 25,24,25
   24 WRITE (6,33)
      STOP
   25 T1=TCON/TCR
      WRITE (6,29) COND(8),TCR,T1
      RETURN
!
   26 PZCR=ROOT
      WRITE (6,30) COND(8),PZCR
      RETURN
!
   27 DIVK=ROOT
      WRITE (6,31) PARAM(JLO+1),PARAM(JUP+1),DIVK
      LSUB=4
!         TO FIND QDIV FOR KDIV FOUND
      KSUB=1
      MSUB=4
      ASSUME=QCON/2.
      STEP=QCON/4.
      RETURN
!
   28 FORMAT (/' 1/R    ', ES16.8/' Q(DIV) ',ES16.8,'   Q/Q(DIV) ',ES16.8)
   29 FORMAT (/' 1/R    ', ES16.8/' TCR    ',ES16.8,'   T/TCR    ',ES16.8)
   30 FORMAT (/' 1/R    ', ES16.8/' PZCR   ',ES16.8)
   31 FORMAT (/' X=', ES13.5,' TO',ES13.5/' K1DIV=',ES16.8)
   32 FORMAT (' Q(DIV) EQUALS ZERO  NO Q1')
   33 FORMAT (' TCR EQUALS ZERO  NO T1')
      END Subroutine Qtpk

      SUBROUTINE PEA (N)
!     THIS SECTION FINDS TOTAL MASS AND INTEGRALS FOR COMPUTING P.
!
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      GO TO (1,6), N
!          INTEGRAL M(X)
    1 JX2=2
      JXL=JXSTA
      JY21=JXSTA+2
      CALL GRAL (PARAM(1:),JX2,JXL,JY21,EMI(1))
!          TOTAL MASS
      JY21=JXSTA+2
      TMASS=SumTrap(PARAM(1:),JX2,JXL,JY21)
!          INTEGRAL D(CD)S/DX
      JY21=4*JXSTA+2
      CALL GRAL (PARAM(1:),JX2,JXL,JY21,DCDI(1))
!          TOTAL INTEGRAL D(CD)S/DX
      JY21=4*JXSTA+2
      TOTD=SumTrap(PARAM,JX2,JXL,JY21)
      TOTD=TOTD+SCDB
!          DIAMETER AND DD/DX
      DO 2 J=1,JXSTA
      K=JNEL+J
    2 DIA(J)=PARAM(K)

      JXM1=JXSTA-1
      DO 4 J=1,JXM1
      IF ((PARAM(J+1)-PARAM(J)) .EQ. 0.0) THEN
        shdcma(j)=0.0
      ELSE
        SHDCMA(J)=(DIA(J+1)-DIA(J))/(PARAM(J+1)-PARAM(J))
      END IF
    4 END DO

      SHDCMA(JXSTA)=SHDCMA(JXM1)
      DO 5 J=1,JXSTA
      K=J+3.*JXSTA
    5 SHDCMA(J)=(DIA(J)*SHDCMA(J)*PARAM(K))/4.
      RETURN
!
!          D
    6 DEE=TOTD*COMPQ
!          T/M(T)
      TMT=(COMPT-DEE)/TMASS
      GO TO (7,7,8,7), KSUB
!          T/M(T)*INTEGRAL M(X),Q*INTEGRAL D(CD)S
    7 CC=COMPT
      GO TO 9
    8 CC=COMPPZ
    9 DO 10 J=1,JXSTA
   10 PSUM(J)=TMT*EMI(J)+COMPQ*DCDI(J)-CC

      RETURN
      END Subroutine PEA

      SUBROUTINE BOUNLO
!     THIS SECTION COMPUTES AT THE LOWER BOUNDARY.
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
!          CLEAR TEMPORARY STORAGE FOR PLUS VALUES
      DO 1 I=1,7
    1 ROWN(I)=0.0
      PROWN=0.0
      JSET=1
      CALL PARMOV
      DO 6 J=1,2
      JBRAN=J
!          SELECT INITIAL CONDITIONS FOR SOLUTION 1 OR 2
!         V(A),ALPHA(A),M(A),1/R
      VZER=COND(4*J-3)
      ALZER=COND(4*J-2)
      EMZER=COND(4*J-1)
      R=COND(4*J)
!              COMPUTE STARTING VECTOR FROM BOUNDARY CONDITIONS
!              DETERMINE SYSTEM
      CALL SYST (2)
!
!         MOVE CHI(A) AND X(A) TO START RECURRENCE SOLUTION
!            CHI(A) TO CHI(N)
!          DETERMINE SOLUTION
      GO TO (2,4), JBRAN
!          SOLUTION 1
    2 DO 3 I=1,8
    3 CHIN1(I,1)=CHIZER(I)
      CYCLE   !!! GO TO 6
!          SOLUTION 2
    4 DO 5 I=1,8
    5 CHIN2(I,1)=CHIZER(I)
    6 END DO
!
!         X(A) TO X(N)
      EXN=EXNP1
!
      GO TO (8,7), KIF
!          FINAL RUN
!          TO OUTPUT ROUTINE
    7 CALL OUTPUT (2)
!          INITIAL RUN
    8 RETURN
      END Subroutine Bounlo

      SUBROUTINE SPAN
!     THIS SECTION COMPUTES ACROSS THE SPAN FOR CONTINUOUS
!     AND DISCONTINUOUS INTERVALS
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
!          BEGIN ALPHA INTEGRATION(SOLUTION 1)
      SUMAL1=0.0
!          BEGIN ALPHA INTEGRATION(SOLUTION 2)
      SUMAL2=0.0
!          BEGIN Y INTEGRAL
      YSUM(1)=YZER
!          BEGIN RECURRENCE SOLUTION
      DO 9 KLOOP=2,JXSTA
      KLOOY=KLOOP
!          MOVE X AND PARAMETERS AT NEXT STATION
      CALL PARMOV
!          TO Y SUBROUTINE
      CALL Y (1,KLOOY)
!
      DO 6 J=1,2
!          SELECTS BRANCH
      JBRAN=J
!          RESET INITIAL VALUES
      VZER=COND(4*J-3)
      ALZER=COND(4*J-2)
      EMZER=COND(4*J-1)
      R=COND(4*J)
!          CONTINUOUS INTERVAL
!              DELTA X
      DELC=ABS(EXNP1-EXN)
      DELC2=DELC/2.0
!          COMPUTE ELEMENTS OF (K)
      CALL CLEMET
!
!          FIND CHI(N+1)
      GO TO (1,3), JBRAN
!          SOLUTION 1
    1 CALL EMULT (CKAY,CHIN1,CHIA,8,8,1)
      DO 2 I=1,8
    2 CHNP11(I,1)=CHIA(I,1)+CHIB(I,1)
!          ALPHA(N+1)+ALPHA(N)
      SUMA=CHNP11(4,1)+CHIN1(4,1)
      GO TO 5
!          SOLUTION 2
    3 CALL EMULT (CKAY,CHIN2,CHIA,8,8,1)
      DO 4 I=1,8
    4 CHNP12(I,1)=CHIA(I,1)+CHIB(I,1)
!          ALPHA(N+1)+ALPHA(N)
      SUMA=CHNP12(4,1)+CHIN2(4,1)
!          TO Y SUBROUTINE
    5 CALL Y (2,KLOOY)
    6 END DO
!
!          TO Y SUBROUTINE
      CALL Y (3,KLOOY)
!          MOVE X(N+1) TO X(N)
      EXN=EXNP1
!          MOVE CHI(N+1) TO CHI(N)
      DO 7 I=1,8
      CHIN1(I,1)=CHNP11(I,1)
      CHIN2(I,1)=CHNP12(I,1)
    7 END DO


!!!      GO TO (9,8), KIF
!!!
!!!          FINAL RUN
!!!    8 CALL OUTPUT (2)
!!!          INITIAL RUN
        IF (kif==2) CALL Output(2)
    9 END DO
      RETURN
      END Subroutine Span

      SUBROUTINE FIND
!     THIS SECTION SEARCHES FOR QDIV,TCR,P(A),OR K1DIV
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
!
!          ITERATION COUNT
    1 ITCNT=0    ! label 1 never used???
      LTST=100
      IF=1
!          ASSUME VALUES OF W UNTIL F(W1) AND F(W2) HAVE UNLIKE SIGNS
!         W=Q,T,P(A),OR K1
!          TO STABILITY EQUATION FOR F(W1)
      CALL SYST (3)
      SAVW(1)=TEMP
      SAVFW(1)=FMEG
      I2CNT=1
!          DETERMINE SIGN OF F(W1)
      IF (SAVFW(1)) 3,2,3
    2 SAVW(4)=SAVW(1)
      GO TO 29
    3 SIGN1=ABS(SAVFW(1))/SAVFW(1)
!          ASSUME NEXT W AND FIND F(W2)
    4 TEMP=SAVW(1)+TEMP2
!          MOVE FOR COMPUTING
      CALL QTPK (2)
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
      CALL SYST (3)
      SAVW(2)=TEMP
      SAVFW(2)=FMEG
!          DETERMINE SIGN OF F(W2)
      IF (SAVFW(2)) 6,5,6
    5 SAVW(4)=SAVW(2)
      GO TO 29
    6 SIGN2=ABS(SAVFW(2))/SAVFW(2)
!          CHECK FOR UNLIKE SIGNS
      IF (SIGN1-SIGN2) 10,7,10
!          LIKE SIGNS,TRY AGAIN
    7 SAVW(1)=SAVW(2)
      SAVFW(1)=SAVFW(2)
      SIGN1=SIGN2
      IF (I2CNT-LTST) 9,8,8
!          NO CROSSING,EXIT FROM LOOP
    8 WRITE (6,33) I2CNT,TEMP,KSUB
      IF (KSUB.EQ.4) LSUB=4
      IF=2
      GO TO 30
    9 I2CNT=I2CNT+1
      GO TO 4
!
!          UNLIKE SIGNS,NARROWINTERVAL
   10 TEMP2=TEMP2/2.0
      TEMP=SAVW(1)+TEMP2
      CALL QTPK (2)
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
      CALL SYST (3)
      SAVW(3)=TEMP
      SAVFW(3)=FMEG
      IF (SAVFW(3)) 12,11,12
!          F(W3)=0
   11 SAVW(4)=SAVW(3)
      GO TO 29
   12 SIGN3=ABS(SAVFW(3))/SAVFW(3)
!          COMPARE F(W3) WITH F(W1)
      IF (SIGN1-SIGN3) 14,13,14
!          LIKE SIGNS,MOVE LOWER LIMIT
   13 SAVW(1)=SAVW(3)
      SAVFW(1)=SAVFW(3)
      SIGN1=SIGN3
      GO TO 15
!          UNLIKE SIGNS,MOVE UPPER LIMIT
   14 SAVW(2)=SAVW(3)
      SAVFW(2)=SAVFW(3)
      SIGN2=SIGN3
   15 IF (TEMP2-EPS1) 16,16,10
!
!          SAVE W3 AND DELTA W USED IN INTERVAL
   16 TINT=TEMP2
      THRINT=SAVW(3)
!          W(N+1)=W(N)-F(W(N))/M
!              M=(F(W1)-F(W2))/(W1-W2).THE SLOPE(M) IS
!              ASSUMED TO BE ALMOST CONSTANT IN THE INTERVAL.
      SLOPE=(SAVFW(1)-SAVFW(2))/(SAVW(1)-SAVW(2))
!          F(W(N))/M
      CHANGE=SAVFW(1)/SLOPE
      TEMPA=ABS(CHANGE)
!          TEST CHANGE FOR .1 OR LESS
      IF (TEMPA-.1) 19,19,17
!          NO---GREATER,GO TO NARROW INTERVAL
   17 EPS1=EPS1/10.0
      IF (EPS1-.0000001) 18,10,10
   18 SAVW(4)=SAVW(3)
      GO TO 29
!
!          W(N)=W1(STARTING VALUE IN INTERVAL)
!              DELTAW= F(W(N))/M
!          BEGINNING DELTA W
   19 TEMP2=CHANGE
      TEMPB=0.0
      SAVW(3)=SAVW(1)
      SAVFW(3)=SAVFW(1)
!
!          BEGIN ITERATION TO FIND W(N+1)
   20 TEMP=SAVW(3)-TEMP2
      CALL QTPK (2)
      IF (TEMP-SAVW(2)) 23,21,21
!          ESTIMATE GREATER THAN UPPER LUMIT
!          PICK UP DELTA W USED IN INTERVAL AND REDUCE FURTHER
   21 TEMP2=TINT
      EPS1=EPS1/10.0
      IF (EPS1-.0000001) 22,10,10
   22 SAVW(4)=THRINT
      GO TO 29
!
   23 ITCNT=ITCNT+1
      IF (ITCNT-75) 25,25,24
!          NON CONVERGENT ROOT.EXIT AFTER 75 TIMES
   24 WRITE (6,32) SAVW(1),SAVW(2),SAVW(3),SAVW(4),TEMP,TEMP2,TEMPA,TEMP&
     &B,TEMPC,EPS1,EPS2,EPS3
      IF=2
      GO TO 30
!
   25 TEMPB=TEMP2
      CALL PEA (2)
      CALL BOUNLO
      CALL SPAN
      CALL SYST (3)
!          W(N+1)
      SAVW(4)=TEMP
      SAVFW(4)=FMEG
!          W(N+1)-W(N)
      TEMPA=ABS(SAVW(4)-SAVW(3))
!          IF LESS THAN EPS2,EXIT
      IF (TEMPA-EPS2) 29,29,26
!          (W(N+1)-W(N))/W(N+1)
   26 TEMPC=TEMPA/ABS(SAVW(4))
!          IF LESS THAN EPS3,EXIT
      IF (TEMPC-EPS3) 29,29,27
!          GREATER,TRY NEXT
   27 TEMP2=SAVFW(4)/SLOPE
      SAVW(3)=SAVW(4)
      SAVFW(3)=SAVFW(4)
!          COMPARE DELTA WITH PREVIOUS DELTA
      IF (TEMPB+TEMP2) 20,28,20
!          IF ZERO,AVERAGE FOR ROOT
   28 SAVW(4)=(SAVW(4)+(SAVW(3)-TEMP2))/2.0
!
!          ROOT FOUND
!          SET BEGINNING FOR NEXT W,THEN EXIT
   29 ROOT=SAVW(4)
   30 CONTINUE
!
   31 RETURN   !!! 31 never used???
!
   32 FORMAT (' NON CONVERGENT ROOT'/(4E16.8))
   33 FORMAT (' NO CROSSING IN ',I6,'  ITERATIONS'/ &
      ' LAST ESTIMATE=',ES16.8, ' FOR SUBCASE I2')
      END Subroutine Find

      SUBROUTINE PARMOV
!     THIS SECTION STORES MINUS VALUES AND MOVES PLUS VALUES TO
!     TEMPORARY BLOCK.
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
!          MOVE INPUT PARAMETERS AND P TO TEMPORARY BLOCK
      J=JSET
      K=(1+JXSTA)+(J-1)*JCR
!          MOVE X
      EXNP1=PARAM(J)
!          MOVE OTHER PARAMETERS
!          ROWN(1)=M,ROWN(2)=EI,ROWN(3)=DCLA
!          ROWN(4)=DCDA,ROWN(5)=K1,ROWN(6)=K2,ROWN(7)=DIA
      DO 1 I=1,JCR
      ROWN(I)=PARAM(K)
    1 K=K+1
      PSHN=SHDCMA(J)
      IF (J-1) 3,2,3
!         P=P(A),IF A STATION
    2 PROWN=COMPPZ
      GO TO 4
    3 PROWN=PSUM(J)
    4 JSET=JSET+1
      RETURN
      END Subroutine Parmov

      SUBROUTINE CLEMET
!     THIS SECTION COMPUTES ELEMENTS OF K MATRIX
!     FOR CONTINUOUS INTERVALS
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      GKA=-ROWN(5)*ROWN(3)*COMPQ
      GB=PROWN
      GKB=PROWN-PSHN*COMPQ
      GKC=-1.0/(ROWN(6)*ROWN(2))
      GKD=DELC2
      GKE=ROWN(1)*EMU**2
      GKK=EXNP1
      GCD=GKC*GKD
      GCD2=GKD*GCD
      GAD=GKA*GKD
      GAD2=GKD*GAD
      GACD=GKA*GCD2
      GBCD=1.0+GKB*GCD2
      GBAD=GKB-GAD
      GDBAD=GKD*GBAD
      G18=GKD*(GKA*ELMBDA*(1.+GB*GCD*GKK)-GKB*GCD*GKE)-GKE
      G28=GKD*G18
      G38=GKD*(GCD*GKE-GB*GKC*GKK*ELMBDA)-ELMBDA
      G48=GKD*G38
      G58=GKD*(GKA*GKD*ELMBDA-GKB*ELMBDA-GKE)+GB*GKK*ELMBDA
      G68=GKD*G58
      G78=0.
      G88=GCD2*GBAD+1.
!          1/D
      DCIP=1.0/(GCD2*GBAD+1.0)
!           COLUMNS 2,4,6,8 OF K
      CKAY(1,2)=GACD
      CKAY(2,2)=GBCD
      CKAY(3,2)=-GCD
      CKAY(4,2)=-GCD2
      CKAY(5,2)=1.0
      CKAY(6,2)=GKD
      CKAY(1,4)=-GKA
      CKAY(2,4)=-GAD
      CKAY(3,4)=-GCD*GBAD
      CKAY(4,4)=1.0
      CKAY(5,4)=GBAD
      CKAY(6,4)=GDBAD
      CKAY(1,6)=GAD*GKC
      CKAY(2,6)=GACD
      CKAY(3,6)=-GKC
      CKAY(4,6)=-GCD
      CKAY(5,6)=-GCD*GBAD
      CKAY(6,6)=1.0
      CKAY(1,8)=G18
      CKAY(2,8)=G28
      CKAY(3,8)=G38
      CKAY(4,8)=G48
      CKAY(5,8)=G58
      CKAY(6,8)=G68
      CKAY(7,8)=G78
      CKAY(8,8)=G88
!           COLUMNS 1,3,5,7 OF K
      DO 1 J=1,7,2
      K=J+1
      DO 1 I=1,8
    1 CKAY(I,J)=GKD*CKAY(I,K)
!          MULTIPLY BY 1/D
      DO 2 I=1,8
      DO 2 J=1,8
    2 CKAY(I,J)=DCIP*CKAY(I,J)
      GO TO (3,5), JBRAN
    3 DO 4 I=1,8
!          SOLUTION 1
    4 CHIB(I,1)=0.0
      RETURN
!          SOLUTION 2
    5 DO 6 I=1,8
    6 CHIB(I,1)=0.
      RETURN
      END Subroutine Clemet

      SUBROUTINE OUTPUT (N)
!     THIS SECTION PRINTS OUTPUT
!
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
!
      GO TO (1,2,3), N
!          HEADINGS
    1 WRITE (6,5)
      IANS=0
      IALPH=JK+JXSTA+1
      RETURN
!          X,VPRIME,V,ALPHA PRIME,ALPHA,M PRIME,M,R PRIME,R
    2 ANS(1)=EXNP1
      ANS(2)=CHIN1(1,1)+CHIN2(1,1)
      ANS(3)=CHIN1(2,1)+CHIN2(2,1)
      ANS(4)=CHIN1(3,1)+CHIN2(3,1)
      ANS(5)=CHIN1(4,1)+CHIN2(4,1)
      ANS(6)=CHIN1(5,1)+CHIN2(5,1)
      ANS(7)=CHIN1(6,1)+CHIN2(6,1)
      ANS(8)=CHIN1(7,1)+CHIN2(7,1)
      ANS(9)=CHIN1(8,1)+CHIN2(8,1)
      WRITE (6,6) IANS,(ANS(I),I=1,7)
!          SAVE ALPHA FOR AERODYNAMIC LIFT FORCE(INTEGRALS)
      PARAM(IALPH)=ANS(5)
      IALPH=IALPH+1
      IANS=IANS+1
      RETURN
!
    3 IANS=0
!
!          X,P,Y
      WRITE (6,7)
      DO 4 I=1,JXSTA
      WRITE (6,6) IANS,PARAM(I),PSUM(I),YSUM(I)
    4 IANS=IANS+1
      RETURN
!
    5 FORMAT(/3X,'STATION',13X,'X',11X,'V PRIME',13X,'V',9X,            &
     & 'ALPHA PRIME',9X,'ALPHA',10X,'M PRIME',12X,'M',7X)
    6 FORMAT(I8,E24.8,6E16.8)
    7 FORMAT(/3X,'STATION',13X,'X',15X,'P',15X,'Y',7X)
      END Subroutine Output

      FUNCTION SumTrap(A,NX2,NXL,NY21) RESULT(f)
      IMPLICIT NONE
      REAL,INTENT(IN),DIMENSION(:):: a
      INTEGER,INTENT(IN):: nx2,nxl
      INTEGER,INTENT(IN OUT):: ny21
      REAL:: f
      REAL:: fsum
      REAL:: h,b
      INTEGER:: i

!!!      DIMENSION A(10200)
!     TOTAL INTEGRAL
      fsum=0.0
      DO I=NX2,NXL
        H=ABS(A(I)-A(I-1))
        B=(A(NY21)+A(NY21-1))/2.0
        fsum=fsum+H*B
        NY21=NY21+1
      END DO
      f=fsum
      RETURN
      END Function SumTrap

      SUBROUTINE GRAL (A,NX2,NXL,NY21,Q000FL)
!     RUNNING INTEGRAL
      DIMENSION Q000FL(600), A(10200)
      M=1
      Q000FL(M)=0.0
      DO I=NX2,NXL
        M=M+1
        H=ABS(A(I)-A(I-1))
        B=(A(NY21)+A(NY21-1))/2.0
        Q000FL(M)=Q000FL(M-1)+H*B
        NY21=NY21+1
      END DO
      RETURN
      END Subroutine Gral

      SUBROUTINE EMULT (A,B,C,NRA,MRB,LCC)
!     MATRIX MULTIPLICATION  
!     same as c(1:nra,1:lcc)=MATMUL(a(1:nra,1:mrb),b(1:mrb,1:lcc))
!     suggests that lcc is always 1, but maybe not...
      DIMENSION A(8,8), B(8,1), C(8,1)
      DO 1 I=1,NRA
      DO 1 J=1,LCC
      C(I,J)=0.0
      DO 1 K=1,MRB
    1 C(I,J)=C(I,J)+A(I,K)*B(K,J)
      RETURN
      END Subroutine Emult

      SUBROUTINE Y (N,K)
!     THIS SECTION FINDS THE VALUE OF Y AND THE INTEGRAL OF ALPHA
! notice that the first variable in COMMON is renamed M, not N
! Probably should give it a totally different name, such as timeThruCounter...
      COMMON M,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
      GO TO (1,2,7), N
!
!          CLEAR TEMPORARY CELLS
    1 ALA=0.0
      ALB=0.0
      ALC=0.0
      ALD=0.0
      RETURN
!
!          PICK SOLUTION AND STORE(ALPHA(N+1)+ALPHA(N))
    2 KY=KCOND(JBRAN)
      GO TO (3,4,5,6), KY
!          V(A)(ALPHA(N)+ALPHA(N+1))
    3 ALA=SUMA
      RETURN
!          ALPHA(A)(ALPHA(N)+ALPHA(N+1))
    4 ALB=SUMA
      RETURN
!          M(A)(ALPHA(N)+ALPHA(N+1))
    5 ALC=SUMA
      RETURN
!         MOVE ALPHA FROM B SOLUTION
!          1/R(ALPHA(N)+ALPHA(N+1))
    6 ALD=SUMA
      RETURN
!
    7 DELO2=ABS(EXNP1-EXN)/2.0
!          Y
!          LAMBDA(X(N+1)+X(N))
      SUMX=ELMBDA*(EXNP1+EXN)
!          SUMMATION(N)TO(N+1)
      SUMA=ALA+ALB+ALC
!          1/R(LAMBDA(X(N+1)+X(N)))
      SUMB=COND(4)*SUMX+COND(8)*SUMX
      SUMC=ALD+SUMB
      SUMA=DELO2*(SUMA+SUMC)
      YSUM(K)=YSUM(K-1)+SUMA
      IF (K.NE.JXSTA) RETURN
      GRALYP=YSUM(K)
      GRALAL=SUMAL1+SUMAL2
      RETURN
      END Subroutine Y

      SUBROUTINE EXTRA
!     THIS SECTION COMPUTES PHYSICAL AND
!     STABILITY CHARACTERISTICS
!
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),CHN&
     &P11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      COMMON /BKA/ KCASE,JNT,KIT,K000FX,KHOM,KIF,TEMP,TEMP2
      COMMON /BKA/ ROOT,JNCOL,JXSTA,QCON,TCON,EMU,PZER,YZER,SMALK,TAU
      COMMON /BKA/ ELMBDA,EN,KSUB,KIA,ASSUME,STEP,EPS1,EPS2,EPS3,TMASS
      COMMON /BKA/ DEBUG,TMT,PROWN,JSET,NA,MA,KBO,KUP,COMPQ,COMPT,COMPPZ
      COMMON /BKA/ QDIV,Q1,TCR,T1,PZCR,DIVK,EXN,EXNP1,JBRAN,DELC,DELC2,A&
     &LPHL
      COMMON /BKA/ SIGN1,SIGN2,SIGN3,ITCNT,FMEG,VZER,ALZER,MZER,R,FM,EMM&
     &N
      COMMON /BKA/ EIMN,CNAMN,FMN,PNUMN,DPNUMN,ZPMN,ETAMN,ONEKMN,TWOKMN
      COMMON /BKA/ PMMN,IANS,IALPH,EMZER,KY,ALA,ALB,ALC,ALD,ALE,SUMA
      COMMON /BKB/ PARAM(10200),EMI(600),DCDI(600),PSUM(600),YSUM(600),D&
     &IA(600),SCDB,SHDCMA(600),PSHN
      COMMON /BKC/ COND(8),KCOND(2),ROWN(7),SAVW(4),SAVFW(4),ANS(9)
      COMMON /BKC/ KADD(11),OUT(10)
      COMMON /KBLK/ JLO,JUP,KLO,KUPP,SUMEK,STEPK,LSUB,SK1,SK2,SK3,DIFF1
      COMMON /KBLK/ DIFF2,DIFF3,SN1,SN2,SN3,EPSK,MSUB,ACOND(8),ACTK,FINK
      COMMON /ROBLK/ JCR,JNEL,JK
      COMMON /JBLK/ IF
      COMMON /ALY/ SUMAL1,SUMAL2,GRALAL,GRALYP
      COMMON /AD1/ TOTD,TOTW,DEE
      COMMON /BLKT/ KTST,LTST
      COMMON /BLKT/ I2CNT
      REAL NEWTER(2000)
!
!          PUT INPUT DATA BACK IN COLUMNS
      LE=JK+2*JXSTA
      DO 1 ME=1,JCR
      JE=JXSTA+ME
      KE=JE+(JXSTA-1)*JCR
      DO 1 IE=JE,KE,JCR
      LE=LE+1
    1 PARAM(LE)=PARAM(IE)
!
!          MOVE ALPHA
      IALPH=JK+JXSTA
      DO 2 ME=1,JXSTA
      JE=IALPH+ME
      LE=LE+1
    2 PARAM(LE)=PARAM(JE)
!          SHIFT COLUMNS TO FOLLOW X
      LE=JK+2*JXSTA+1
      JE=LE+JK-1
      KE=JXSTA
      DO 3 ME=LE,JE
      KE=KE+1
    3 PARAM(KE)=PARAM(ME)
      IF (LSUB.NE.4) GO TO 5
!          K1=ACTUAL K FOR EFFECTIVE STATIC MARGIN
      LE=5*JXSTA+JLO+1
      ME=5*JXSTA+JUP+1
!          INTERVAL IN K1
      DO 4 KE=LE,ME
    4 PARAM(KE)=ACTK
!
!          (X)*(M),(X**2)*(M)
    5 LE=14*JXSTA
      ME=15*JXSTA
      DO 6 IE=1,JXSTA
      JE=JXSTA+IE
      LE=LE+1
      ME=ME+1
      PARAM(LE)=PARAM(IE)*PARAM(JE)
    6 PARAM(ME)=PARAM(IE)*PARAM(LE)
!
!       CENTER OF GRAVITY
      JX2=2
      JY21=14*JXSTA+2
      ATOP=SumTrap(PARAM,JX2,JXSTA,JY21)
      EXBAR=ATOP/TMASS
!
!          MASS MOMENT OF INERTIA
      JY21=15*JXSTA+2
      AJAY=SumTrap(PARAM,JX2,JXSTA,JY21)
!
!          RADIUS OF GYRATION
      RBAR=SQRT(AJAY/TMASS)
!
!          MASS MOMENT OF INERTIA ABOUT THE CENTROID
      AJAYZ=AJAY-TMASS*EXBAR**2
!
!         (K1)*(DCLA), (K1)*(DCLA)*X
      LE=14*JXSTA
      ME=15*JXSTA
      DO 7 IE=1,JXSTA
      JE=3*JXSTA+IE
      NE=5*JXSTA+IE
      LE=LE+1
      ME=ME+1
!         (K1)*(DCLA)
      PARAM(LE)=PARAM(NE)*PARAM(JE)
!         (K1)*(DCLA)*X
    7 PARAM(ME)=PARAM(IE)*PARAM(LE)
!
!          TOTAL AERODYNAMIC LIFT COEFFICIENTS
      JY21=14*JXSTA+2
      CNAS=SumTrap(PARAM,JX2,JXSTA,JY21)
!
!          AERODYNAMIC CENTER OF PRESSURE
      JY21=15*JXSTA+2
      TOP=SumTrap(PARAM,JX2,JXSTA,JY21)
      DO 8 I=1,JXSTA
    8 NEWTER(I)=PARAM(I)
      DO 9 I=1,JXSTA
      K=JXSTA+I
    9 NEWTER(K)=SHDCMA(I)
      K=JXSTA+2
      TOTNEW=SumTrap(NEWTER,JX2,JXSTA,K)
      EXR=(TOP-TOTNEW)/CNAS
!          SAVE INTEGRALS FOR SHORT PERIOD FREQUENCY
!         (INT K1*DCLA)
      BTOP=CNAS
!         (INT K1*DCLA*X)
      CTOP=TOP
!
!          STATIC MARGIN
      SMR=EXBAR-EXR
!
!          Q*ALPHA
      DO 10 IE=1,JXSTA
      ME=13*JXSTA+IE
      JE=7*JXSTA+IE
   10 PARAM(ME)=QCON*PARAM(JE)
      LEE=16*JXSTA
!          AERODYNAMIC LIFT FORCE
      DO 11 IE=1,JXSTA
      LE=14*JXSTA+IE
      ME=13*JXSTA+IE
      LEE=LEE+1
   11 PARAM(LEE)=PARAM(LE)*PARAM(ME)
!
      WRITE (6,18)
      WRITE (6,19) TMASS,EXBAR,CNAS,EXR,RBAR,SMR,AJAY,AJAYZ
!
!          SHORT PERIOD FREQUENCY
!          (INT M)*(INT MX**2)-(INT MX)**2
      DEN=(TMASS*AJAY)-(ATOP*ATOP)
!         (INT DCLA)*(INT MX)-(INT X*DCLA)*(INT M)
      ANUM=(BTOP*ATOP)-(CTOP*TMASS)
      DEN=QCON*(ANUM/DEN)
      ANUM=((TCON+(QCON*BTOP))/((2.0*EMU)*TMASS))**2
      SA=DEN-ANUM
      IF (SA) 13,12,12
   12 SB=SQRT(SA)
      FR=(1.0/6.2831854)*SB
      WRITE (6,20) SB,FR
      IF (LSUB.EQ.4) GO TO 14
      RETURN
!          EXIT IF NEGATIVE ARGUMENT FOR SQ.ROOT
   13 WRITE (6,21) SA
      IF (LSUB.EQ.4) GO TO 14
      RETURN
!
!         EFFECTIVE STATIC MARGIN(K DIVERGENCE ONLY)
   14 KXL=JLO+1
      KXU=JUP+1
!         DCLA(XBAR)
      DO 15 IE=KXL,KXU
      JE=3*JXSTA+IE
      LE=15*JXSTA+IE
      PARAM(LE)=PARAM(JE)*(EXBAR-PARAM(IE))
   15 END DO
!         (INT DCLA(XBAR))
      JX2=KXL+1
      JY21=15*JXSTA+JX2
      DTOP=SumTrap(PARAM,JX2,KXU,JY21)
      IF (I2CNT.GE.LTST) GO TO 17
   16 SMEF=((ACTK-DIVK)*DTOP-TOTNEW)/BTOP
      WRITE (6,22) ACTK,DIVK,DTOP,TOTNEW,BTOP
      WRITE (6,23) SMEF
      RETURN
   17 IF (MSUB.EQ.4) GO TO 16
      WRITE (6,24)
      RETURN
!
   18 FORMAT(/5X,'MASS',12X,'X(CG)',11X,'C(LA)',11X,'X(CP)',11X,'RBAR', &
     & 10X, 'X(SM,RIG)',11X,'J',12X,'JZERO',6X)
   19 FORMAT(8E16.8)
   20 FORMAT(/' OMEGA ',E16.8/' SHORT PERIOD FREQUENCY ',E16.8)
   21 FORMAT(/' NO SHORT PERIOD FREQUENCY'/' SQ.ROOT OF ',E16.8)
   22 FORMAT(/' K1 =',E15.7,' K1 DIV=',E15.7,' DCL(XC-X)=',E15.7,       &
     & ' SH  DCMA =',E15.7,' K1DCL =',E15.7)
   23 FORMAT(/'         X(GSM)'/4X,E16.8)
   24 FORMAT(/' NO K DIV ,NO X(GSM)')
      END Subroutine Extra

END Module AeroelasticDivergenceProcedures




      PROGRAM AeroelasticDivergence
USE AeroelasticDivergenceProcedures
      COMMON N,JCASE,D1(8,8),D2(8,1),CHIZER(8),CHIN1(8,1),CHIN2(8,1),    &
     & CHNP11(8,1),CHNP12(8,1),CHIA(8,1),CHIB(8,1),CKAY(8,8)
      INTEGER:: errCode
      CHARACTER(LEN=80):: fileName
      CHARACTER(LEN=*),PARAMETER:: VERSION = '0.6 (4 February 2010)'
!----------------------------------------------------------------------------
      N=1
  WRITE(*,*) 'Diverge - '
  WRITE(*,*) 'Version '//VERSION
  DO
    WRITE(*,*) 'Enter the name of the input file: '
    READ(*,'(A)') fileName
    IF (LEN_TRIM(fileName)==0) STOP
    OPEN(UNIT=5,FILE=fileName,STATUS='OLD',IOSTAT=errCode,ACTION='READ')
    IF (errCode==0) EXIT
    WRITE(*,*) 'Unable to open this file. Try again.'
  END DO
  OPEN(UNIT=6,FILE='diverge.out',STATUS='REPLACE',ACTION='WRITE')
  OPEN(UNIT=2,FILE='diverge.tp2',STATUS='REPLACE',                       &
    FORM='UNFORMATTED',ACTION='READWRITE')
 
  DO
    CALL Enput()
    CALL R0863()
    WRITE(6,*) "Case completed. Look for more."
  END DO
END Program AeroelasticDivergence   ! =======================================


